@@ -36,6 +36,8 @@ from functools import partial
 import requests
 import numpy as np
 
+from Bio import Entrez
+
 from cblaster import database
 from cblaster.classes import Organism, Scaffold, Subject
 
@@ -43,7 +45,7 @@ from cblaster.classes import Organism, Scaffold, Subject
 LOG = logging.getLogger(__name__)
 
 
-def efetch_IPGs(ids, output_handle=None):
+def efetch_IPGs(ids, output_file=None):
     """Queries the Identical Protein Groups (IPG) resource for given IDs.
 
     The NCBI caps Efetch requests at 10000 maximum returned records (retmax=10000)
@@ -52,39 +54,36 @@ def efetch_IPGs(ids, output_handle=None):
 
     Args:
         ids (list): Valid NCBI sequence identifiers.
-        output_handle (file handle): File handle to write to.
+        output_file (str): File to write the ipg table to.
     Returns:
         List of rows from resulting IPG table, split by newline.
     """
-
-    # Split into chunks since retmax=10000
-    chunks = [ids[i: i + 10000] for i in range(0, len(ids), 10000)]
-
-    table = ""
-    for ix, chunk in enumerate(chunks, 1):
-        response = requests.post(
-            "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?",
-            params={
-                "db": "protein",
-                "rettype": "ipg",
-                "retmode": "text",
-                "retmax": 10000,
-            },
-            data={"id": ",".join(chunk)},
-        )
-
-        if response.status_code != 200:
-            raise requests.HTTPError(
-                f"Error fetching sequences from NCBI [code {response.status_code}]."
+    if not ids:
+        raise ValueError("No ids specified")
+    table = []
+    for start in range(0, len(ids), 10000):
+        chunk = ids[start: start + 10000]
+        try:
+            fetch = Entrez.efetch(
+                "protein",
+                rettype="ipg",
+                retmode="text",
+                id=chunk,
+                retmax=10000,
             )
+        except IOError:
+            LOG.exception("Network error while retrieving genomic context")
+            raise
+        lines = [line.decode() for line in fetch.readlines()]
+        table.extend(lines)
 
-        table += response.text
+    if output_file:
+        LOG.info("Writing IPG table to %s", output_file)
+        with open(output_file, "w") as fp:
+            for line in table:
+                fp.write(line)
 
-    if output_handle:
-        LOG.info("Writing IPG table to %s", output_handle.name)
-        output_handle.write(table)
-
-    return table.split("\n")
+    return table
 
 
 def parse_IP_groups(results):
@@ -114,10 +113,15 @@ def parse_IP_groups(results):
     Entry = namedtuple("Entry", fields)
     groups = defaultdict(list)
     for line in results:
-        if not line or line.startswith("Id\tSource") or line.isspace():
+        if not line or line.startswith("Id\tSource") or line.isspace() \
+                or "skipping" in line:
             continue
         ipg, *fields = line.strip("\n").split("\t")
-        entry = Entry(*fields)
+        try:
+            entry = Entry(*fields)
+        except ValueError:
+            LOG.warning("Failed to parse row in IPG table: %s", fields)
+            continue
         groups[ipg].append(entry)
     return groups
 
@@ -159,11 +163,12 @@ def parse_IPG_table(results, hits):
     This function:
     1. Parses entries from the table, grouped by IPG number with parse_IP_groups()
     2. For each group:
-       a) Find Hit objects linked to any member of the group with find_IPG_hits()
-       b) For each group member, create a Subject object, then place it on its
-          corresponding Scaffold and Organism objects (creating new objects when new
-          scaffolds and organisms are encountered)
-       c) Add Hit objects to every Subject object in the group
+
+    a) Find Hit objects linked to any member of the group with find_IPG_hits()
+    b) For each group member, create a Subject object, then place it on its
+       corresponding Scaffold and Organism objects (creating new objects when new
+       scaffolds and organisms are encountered)
+    c) Add Hit objects to every Subject object in the group
 
     Args:
         results (list): Results from IPG search.
@@ -224,7 +229,7 @@ def parse_IPG_table(results, hits):
                 ipg=ipg,
                 end=int(entry.end),
                 start=int(entry.start),
-                strand=entry.strand,
+                strand=1 if entry.strand == "+" else -1,
             )
 
             organisms[org][st].scaffolds[acc].subjects.append(subject)
@@ -259,7 +264,7 @@ def find_identifier(qualifiers):
 def query_local_DB(hits, db):
     """Queries a local SQLite3 database created using the makedb module.
     """
-    organisms = defaultdict(dict)
+    organisms = {}
     hit_dict = defaultdict(list)
     for hit in hits:
         hit_dict[hit.subject].append(hit)
@@ -271,7 +276,7 @@ def query_local_DB(hits, db):
         strand,
         scaffold,
         organism
-    ) in database.query_database(list(hit_dict), db):
+    ) in database.query_genes(list(hit_dict), db):
         if organism not in organisms:
             organisms[organism] = Organism(organism, "")
         if scaffold not in organisms[organism].scaffolds:
@@ -280,31 +285,49 @@ def query_local_DB(hits, db):
         for hit in hits:
             hit.subject = name
         subject = Subject(
+            id=rowid,
             name=name,
             hits=hits,
             start=int(start_pos),
             end=int(end_pos),
-            strand="+" if strand == 1 else "-"
+            strand=strand,
         )
         organisms[organism].scaffolds[scaffold].subjects.append(subject)
     return [organism for organism in organisms.values()]
 
 
-def cluster_satisfies_conditions(cluster, require=None, unique=3, minimum=3):
+def cluster_satisfies_conditions(
+    cluster,
+    queries,
+    require=None,
+    unique=3,
+    minimum=3,
+    percentage=50,
+):
     """Tests if a cluster of Subjects meets query conditions.
 
     Finds all unique query sequences in hits, then returns True if total number is above
     unique threshold, and any required queries are represented.
     """
-    queries = set(hit.query for subject in cluster for hit in subject.hits)
+    hit_queries = set(hit.query for subject in cluster for hit in subject.hits)
+    hit_overlap = hit_queries.intersection(queries)
     return (
-        len(cluster) >= minimum
-        and len(queries) >= unique
-        and (queries.issuperset(require) if require else True)
+        len(cluster) >= minimum  # Minimum hits
+        and len(hit_queries) >= unique  # Unique hits
+        and (hit_queries.issuperset(require) if require else True)  # Required queries
+        and len(hit_overlap) / len(queries) >= percentage / 100 # Min % of queries
     )
 
 
-def find_clusters(subjects, require=None, unique=3, min_hits=3, gap=20000):
+def find_clusters(
+    subjects,
+    require=None,
+    unique=3,
+    min_hits=3,
+    gap=20000,
+    percentage=50,
+    queries=None
+):
     """Finds clusters of Hit objects matching user thresholds.
 
     Args:
@@ -316,20 +339,13 @@ def find_clusters(subjects, require=None, unique=3, min_hits=3, gap=20000):
     Returns:
         Clusters of Hit objects.
     """
-    if unique < 0 or min_hits < 0 or gap < 0:
-        raise ValueError("Expected positive integer")
-
-    total_subjects = len(subjects)
-
-    if total_subjects < unique:
-        return []
-
-    if total_subjects == 1:
-        if unique == 1 or min_hits == 1:
-            return [subjects]
-        return []
+    if any(value < 0 for value in [min_hits, gap, unique, percentage]):
+        raise ValueError("Expected positive integers")
+    if percentage > 100:
+        raise ValueError("Expected percentage between 0 and 100")
 
     sorted_subjects = sorted(subjects, key=attrgetter("start"))
+
     first = sorted_subjects.pop(0)
     group, border = [first], first.end
 
@@ -337,7 +353,9 @@ def find_clusters(subjects, require=None, unique=3, min_hits=3, gap=20000):
         cluster_satisfies_conditions,
         require=require,
         unique=unique,
-        minimum=min_hits
+        minimum=min_hits,
+        percentage=percentage,
+        queries=queries,
     )
 
     for subject in sorted_subjects:
@@ -393,7 +411,8 @@ def find_clusters_in_organism(
     gap=20000,
     require=None,
     remote=True,
-    query_sequence_order=None
+    query_sequence_order=None,
+    percentage=50
 ):
     """Runs find_clusters() on all scaffolds in an organism."""
     for scaffold in organism.scaffolds.values():
@@ -403,6 +422,8 @@ def find_clusters_in_organism(
             min_hits=min_hits,
             gap=gap,
             require=require,
+            percentage=percentage,
+            queries=query_sequence_order,
         )
         scaffold.add_clusters(clusters, query_sequence_order=query_sequence_order)
         LOG.debug(
@@ -424,6 +445,7 @@ def filter_session(
     unique=3,
     min_hits=3,
     require=None,
+    percentage=50,
 ):
     """Filter a Session object with new thresholds.
 
@@ -436,17 +458,22 @@ def filter_session(
                     hit
                     for hit in subject.hits
                     if (
-                        hit.identity > min_identity
-                        and hit.coverage > min_coverage
-                        and hit.evalue < max_evalue
+                        hit.identity >= min_identity if hit.identity else True
+                        and hit.coverage >= min_coverage if hit.coverage else True
+                        and hit.evalue <= max_evalue
                     )
                 ]
+                if not subject.hits:
+                    scaffold.remove_subject(subject)
+                clusters = [scaffold.subjects]
             clusters = find_clusters(
                 scaffold.subjects,
+                queries=session.queries,
                 gap=gap,
                 min_hits=min_hits,
                 require=require,
                 unique=unique,
+                percentage=percentage,
             )
             scaffold.clusters = []
             scaffold.add_clusters(clusters, query_sequence_order=session.queries)
@@ -504,8 +531,9 @@ def search(
     min_hits=3,
     gap=20000,
     require=None,
+    ipg_file=None,
     query_sequence_order=None,
-    ipg_file=None
+    percentage=None,
 ):
     """Gets the genomic context for a collection of Hit objects.
 
@@ -522,11 +550,13 @@ def search(
 
     Args:
         hits (list): Collection of Hit objects to find clusters in.
+        sqlite_db (str): path to sqlite database.
         require (list): Names of query sequences that must be represented in a cluster.
         unique (int): Unique query sequence threshold.
         min_hits (int): Minimum number of hits in a hit cluster.
         gap (int): Maximum intergenic distance (bp) between any two hits in a cluster.
         query_sequence_order (list): list of sequences of the order in the query file, is
+        ipg_file (str): file to save the ipg table into.
     Returns:
         Dictionary of Organism objects keyed on species name.
     """
@@ -536,7 +566,7 @@ def search(
     else:
         rows = efetch_IPGs(
             [hit.subject for hit in hits],
-            output_handle=ipg_file
+            output_file=ipg_file
         )
         organisms = parse_IPG_table(rows, hits)
 
@@ -550,6 +580,7 @@ def search(
             require=require,
             query_sequence_order=query_sequence_order,
             remote=sqlite_db is None,
+            percentage=percentage,
         )
 
     return organisms
