@@ -5,13 +5,13 @@ This module handles creation of local JSON databases for non-NCBI lookups.
 import logging
 import subprocess
 import sqlite3
+import functools
 
 from pathlib import Path
 from multiprocessing import Pool
 
-from cblaster import helpers
+from cblaster import helpers, sql
 from cblaster import genome_parsers as gp
-from cblaster.sql import FASTA, INSERT, QUERY, SCHEMA
 
 
 LOG = logging.getLogger("cblaster")
@@ -35,8 +35,8 @@ def init_sqlite_db(path, force=False):
             raise FileExistsError(f"File {path} already exists but force=False")
     else:
         LOG.info("Initialising cblaster SQLite3 database to %s", path)
-    with sqlite3.connect(path) as con:
-        con.executescript(SCHEMA)
+    with sqlite3.connect(str(path)) as con:
+        con.executescript(sql.SCHEMA)
 
 
 def seqrecords_to_sqlite(tuples, database):
@@ -47,9 +47,9 @@ def seqrecords_to_sqlite(tuples, database):
         database (str): Path to SQLite3 database
     """
     try:
-        with sqlite3.connect(database) as con:
+        with sqlite3.connect(str(database)) as con:
             cur = con.cursor()
-            cur.executemany(INSERT, tuples)
+            cur.executemany(sql.INSERT, tuples)
     except sqlite3.IntegrityError:
         LOG.exception("Failed to insert %i records", len(tuples))
 
@@ -61,13 +61,26 @@ def sqlite_to_fasta(path, database):
         path (str): Path to output FASTA file
         database (str): Path to SQLite3 database
     """
-    with sqlite3.connect(database) as con, open(path, "w") as fasta:
+    with sqlite3.connect(str(database)) as con, open(path, "w") as fasta:
         cur = con.cursor()
-        for (record,) in cur.execute(FASTA):
+        for (record,) in cur.execute(sql.FASTA):
             fasta.write(record)
 
 
-def query_database(ids, database):
+def _query(query, database, values=None, fetch="all"):
+    with sqlite3.connect(str(database)) as con:
+        cur = con.cursor()
+        query = cur.execute(query, values) if values else cur.execute(query)
+        return query.fetchall() if fetch == "all" else query.fetchone()
+
+
+def query_sequences(ids, database):
+    inner = ", ".join(str(idx) for idx in ids)
+    query = sql.SEQUENCE_QUERY.format(inner)
+    return _query(query, database)
+
+
+def query_genes(ids, database):
     """Queries the cblaster SQLite3 database for a collection of gene IDs.
 
     Args:
@@ -76,15 +89,39 @@ def query_database(ids, database):
     Returns:
         list: Result tuples returned by the query
     """
-    marks = ", ".join("?" for _ in ids)
-    query = QUERY.format(marks)
-    with sqlite3.connect(database) as con:
-        cur = con.cursor()
-        return cur.execute(query, ids).fetchall()
+    inner = ", ".join(str(idx) for idx in ids)
+    query = sql.GENE_QUERY.format(inner)
+    return _query(query, database)
+
+
+def query_intermediate_genes(
+    names, start, end, scaffold, organism, database, local=False
+):
+    """Queries the cblaster SQLite3 database for a collection of intermediate genes.
+
+    These are the genes between start and stop that are not part of the names list
+
+    Args:
+        names (list): a list of names that are part of one cluster
+        start (int): the minimal start a gene can have to be considered intermediate
+        end (int): the maximum end a gene can have to be considered intermediate
+        database (str): Path to SQLite3 database
+    Returns:
+        list: Result tuples returned by the query
+    """
+    marks = ", ".join("?" for _ in names)
+    query = sql.INTERMEDIATE_GENES_QUERY.format(marks)
+    return _query(query, database, values=[*names, scaffold, organism, start, end])
+
+
+def query_nucleotides(scaffold, organism, start, end, database):
+    """Queries a database for a """
+    query = sql.SCAFFOLD_QUERY.format(start, end - start)
+    return _query(query, database, values=[scaffold, organism], fetch="one")
 
 
 def diamond_makedb(fasta, name):
-    """Builds a DIAMOND database from JSON.
+    """Builds a DIAMOND database
 
     Args:
         fasta (str): Path to FASTA file containing protein sequences.
@@ -92,7 +129,7 @@ def diamond_makedb(fasta, name):
     """
     diamond = helpers.get_program_path(["diamond", "diamond-aligner"])
     subprocess.run(
-        [diamond, "makedb", "--in", fasta, "--db", name],
+        [diamond, "makedb", "--in", str(fasta), "--db", name],
         stdout=subprocess.DEVNULL,
         stderr=subprocess.DEVNULL,
     )
@@ -140,29 +177,35 @@ def makedb(paths, database, force=False, cpus=None, batch=None):
         else:
             raise RuntimeError("Existing files found but force=False")
 
-    LOG.info("Initialising SQLite3 database at %s", sqlite_path)
     init_sqlite_db(sqlite_path, force=force)
 
     paths = gp.find_files(paths)
+    if len(paths) == 0:
+        raise RuntimeError("No valid files provided expected genbank, embl or gff with accompanying fasta file.")
     total_paths = len(paths)
     if batch is None:
         batch = total_paths
     path_groups = [paths[i : i + batch] for i in range(0, total_paths, batch)]
+
     LOG.info(
         "Parsing %i genome files, in %i batches of %i",
         total_paths,
         len(path_groups),
         batch,
     )
-
-    with Pool(cpus) as pool:
-        for index, group in enumerate(path_groups, 1):
-            LOG.info("Batch %i: %s", index, [str(p) for p in group])
-            organisms = pool.map(gp.parse_file, group)
-            tuples = gp.organisms_to_tuples(organisms)
-
-            LOG.info("Saving %i genes", len(tuples))
-            seqrecords_to_sqlite(tuples, sqlite_path)
+    try:
+        func = functools.partial(gp.parse_file, to_tuples=True)
+        with Pool(cpus) as pool:
+            for index, group in enumerate(path_groups, 1):
+                LOG.info("Processing batch %i", index)
+                tuples = []
+                for organism in pool.imap(func, group):
+                    for records in organism["records"]:
+                        tuples.extend(records)
+                LOG.info("Saving %i genes", len(tuples))
+                seqrecords_to_sqlite(tuples, sqlite_path)
+    except Exception:
+        LOG.error("File parsing failed, exiting...", exc_info=True)
 
     LOG.info("Writing FASTA to %s", fasta_path)
     sqlite_to_fasta(fasta_path, sqlite_path)
